
-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("PrivateChat")
local tea = AceLibrary("TEA-1.0")
local md5 = AceLibrary("MD5-1.0")

local _G = getfenv(0)

local PROTOCOL_VERSION = 1
local DATABASE_VERSION = 1

-------------------------------------------------------------------------------
-- Localization
-------------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Channel name missing"] = true,
	["Channel name, slash or key missing"] = true,
	["Create Channel"] = true,
	["Created virtual channel |cff0099ff%s|r with slash command |cff0099ff%s|r"] = true,
	["Creates virtual communication channel."] = true,
	["Delete Channel"] = true,
	["Deleted virtual channel |cff0099ff%s|r with slash command |cff0099ff%s|r"] = true,
	["Deletes virtual communication channel."] = true,
	["List Channels"] = true,
	["Lists virtual communication channels."] = true,
	["Slash command |cff0099ff%s|r is already being used"] = true,
	["Virtual channel |cff0099ff%s|r already exists"] = true,
	["Virtual channel |cff0099ff%s|r doesnt exist"] = true,
	["Virtual channels:"] = true,
} end)

-------------------------------------------------------------------------------
-- Main
-------------------------------------------------------------------------------

PrivateChat = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceHook-2.1", "AceComm-2.0")

PrivateChat.defaults = {
	channels = {},
}

PrivateChat.options = {
	type = 'group',
	args = {
		create = {
			type = "text",
			name = L["Create Channel"],
			desc = L["Creates virtual communication channel."],
			usage = "<name> <slash> <key>",
			input = true,
			get = false,
			set = "Create",
		},
		delete = {
			type = "text",
			name = L["Delete Channel"],
			desc = L["Deletes virtual communication channel."],
			usage = "<name>",
			input = true,
			get = false,
			set = "Delete",
		},
		list = {
			type = "execute",
			name = L["List Channels"],
			desc = L["Lists virtual communication channels."],
			func = "List",
		},
	},
}

PrivateChat.revision = tonumber(string.sub("$Revision: 17059 $", 12, -3))
PrivateChat:RegisterDB("PrivateChatDB", "PrivateChatDBPC")
PrivateChat:RegisterDefaults("profile", PrivateChat.defaults)
PrivateChat:SetCommPrefix("PrivateChat" .. PROTOCOL_VERSION)
PrivateChat:RegisterChatCommand( { "/pc", "/PrivateChat" }, PrivateChat.options)

function PrivateChat:OnInitialize()
	self.version = self.version .. " |cffff8888r".. self.revision .."|r" 

	if not self.db.profile.version or self.db.profile.version < DATABASE_VERSION then
		self.db.profile.channels = {}
		self.db.profile.version = DATABASE_VERSION
	end

	self.registered = {}
end

function PrivateChat:OnEnable()
	self:Hook("SendChatMessage")
	self:SecureHook("FCF_SetChatTypeColor")
	self:SecureHook("FCF_CancelFontColorSettings")
	self:SecureHook("FCFMessageTypeDropDown_OnClick")

	self:RegisterComm("PrivateChat" .. PROTOCOL_VERSION, "GUILD")

	for id, _ in pairs(self.db.profile.channels) do
		self:RegisterChannel(id)
	end
end

function PrivateChat:OnDisable()
	for id, _ in pairs(self.db.profile.channels) do
		self:UnregisterChannel(id)
	end
end

-------------------------------------------------------------------------------
-- Commands
-------------------------------------------------------------------------------

function PrivateChat:Create(name, slash, key)
	if not name or not slash or not key then
		self:Print(L["Channel name, slash or key missing"])
		return
	end

	local id = self:NameToID(name)
	local n = string.upper(string.sub(name, 1, 1)) .. string.lower(string.sub(name, 2))

	if self.db.profile.channels[id] then
		self:Print(string.format(L["Virtual channel |cff0099ff%s|r already exists"], n))
		return
	end

	if string.sub(slash, 1, 1) ~= "/" then
		slash = "/" .. slash
	end

	if not self:IsSlashAvailable(slash) then
		self:Print(string.format(L["Slash command |cff0099ff%s|r is already being used"], slash))
		return
	end

	self:CreateChannel(id, n, slash, key)
	self:RegisterChannel(id)
	self:Print(string.format(L["Created virtual channel |cff0099ff%s|r with slash command |cff0099ff%s|r"], n, slash))
end

function PrivateChat:Delete(name)
	if not name then
		self:Print(L["Channel name missing"])
		return
	end

	local id = self:NameToID(name)
	local n = string.upper(string.sub(name, 1, 1)) .. string.lower(string.sub(name, 2))

	if not self.db.profile.channels[id] then
		self:Print(string.format(L["Virtual channel |cff0099ff%s|r doesnt exist"], n))
		return
	end

	if self.registered[id] then
		self:UnregisterChannel(id)
	end

	local slash = self.db.profile.channels[id].slash

	self:RemoveChannel(id)
	self:Print(string.format(L["Deleted virtual channel |cff0099ff%s|r with slash command |cff0099ff%s|r"], n, slash))
end

function PrivateChat:List()
	self:Print(L["Virtual channels:"])
	local i = 0

	for _, channel in pairs(self.db.profile.channels) do
		local c = string.format("%02x", channel.r * 255) .. string.format("%02x", channel.g * 255) .. string.format("%02x", channel.b * 255)
		i = i + 1
		self:Print(string.format("%d. |cff%s%s|r ( |cff0099ff%s|r )", i, c, channel.name, channel.slash))
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function PrivateChat:SendChatMessage(text, chattype, language, destination)
	if self.registered[chattype] then
		text = string.gsub(text, "%%t", UnitName("target") or "<no target>")

		self:RiseChatEvent(chattype, text, UnitName("player"), self.db.profile.channels[chattype].name)

		self:SendCommMessage("GUILD", "MESSAGE", {
			i = chattype,
			t = tea:Encrypt(text, self.db.profile.channels[chattype].key),
		})

		return
	end

	self.hooks.SendChatMessage(text, chattype, language, destination)
end

function PrivateChat:FCF_SetChatTypeColor()
	if self.registered[UIDROPDOWNMENU_MENU_VALUE] and self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE] then
		local r, g, b = ColorPickerFrame:GetColorRGB()

		self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE].r = r;
		self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE].g = g;
		self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE].b = b;

		ChatTypeInfo[UIDROPDOWNMENU_MENU_VALUE].r = r;
		ChatTypeInfo[UIDROPDOWNMENU_MENU_VALUE].g = g;
		ChatTypeInfo[UIDROPDOWNMENU_MENU_VALUE].b = b;
	end
end

function PrivateChat:FCF_CancelFontColorSettings(prev)
	if prev.r and self.registered[UIDROPDOWNMENU_MENU_VALUE] and self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE] then
		self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE].r = prev.r;
		self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE].g = prev.g;
		self.db.profile.channels[UIDROPDOWNMENU_MENU_VALUE].b = prev.b;

		ChatTypeInfo[UIDROPDOWNMENU_MENU_VALUE].r = prev.r;
		ChatTypeInfo[UIDROPDOWNMENU_MENU_VALUE].g = prev.g;
		ChatTypeInfo[UIDROPDOWNMENU_MENU_VALUE].b = prev.b;
	end
end

function PrivateChat:FCFMessageTypeDropDown_OnClick()
	if self.registered[this.value] and self.db.profile.channels[this.value] then
		local f = FCF_GetCurrentChatFrame():GetName()

		if UIDropDownMenuButton_GetChecked() then
			for k, v in self.db.profile.channels[this.value].frames do
				if v == f then
					self.db.profile.channels[this.value].frames[k] = nil
					break
				end
			end
		else
			table.insert(self.db.profile.channels[this.value].frames, f)
		end
	end
end

-------------------------------------------------------------------------------
-- Chat Internals
-------------------------------------------------------------------------------

function PrivateChat:NameToID(name)
	 local t = md5:MD5AsTable(string.lower(name))

	 return string.format("%08x", bit.bxor(t[1], t[2], t[3], t[4]))
end

function PrivateChat:IsSlashAvailable(slash)
	for k, v in pairs(_G) do
		if string.sub(k, 1, 6) == "SLASH_" and v == slash then
			return false
		end
	end

	return true
end

function PrivateChat:CreateChannel(id, name, slash, key)
	self.db.profile.channels[id] = {
		id = id,
		name = name,
		slash = slash,
		key = tea:GenerateKey(key),
		r = 0.45,
		g = 0.90,
		b = 1.00,
		frames = {
			"ChatFrame1"
		},
	}
end

function PrivateChat:RemoveChannel(id)
	self.db.profile.channels[id] = nil
end

function PrivateChat:RegisterChannel(id)
	if not self.registered[id] then
		local channel = self.db.profile.channels[id]

		_G["SLASH_" .. id .. "1"] = channel.slash
		_G["CHAT_MSG_" .. id] = channel.name
		_G["CHAT_" .. id .. "_SEND"] = channel.name .. ": "
		_G["CHAT_" .. id .. "_GET"] = "%s: "

		ChatTypeInfo[id] = { sticky = 1, r = channel.r, g = channel.g, b = channel.b }
		ChatTypeGroup[id] = { "CHAT_MSG_" .. id }

		table.insert(ChannelMenuChatTypeGroups, id)

		for _, f in ipairs(self.db.profile.channels[id].frames) do
			table.insert(_G[f].messageTypeList, id)
		end

		self.registered[id] = 1
	end
end

function PrivateChat:UnregisterChannel(id)
	if self.registered[id] then
		local channel = self.db.profile.channels[id]

		_G["SLASH_" .. id .. "1"] = nil
		_G["CHAT_MSG_" .. id] = nil
		_G["CHAT_" .. id .. "_SEND"] = nil
		_G["CHAT_" .. id .. "_GET"] = nil

		ChatTypeInfo[id] = nil
		ChatTypeGroup[id] = nil

		for k, v in pairs(ChannelMenuChatTypeGroups) do
			if v == id then
				ChannelMenuChatTypeGroups[k] = nil
				break
			end
		end

		for i = 1, NUM_CHAT_WINDOWS, 1 do
			for k, v in pairs(_G["ChatFrame" .. i].messageTypeList) do
				if (v == id) then
					_G["ChatFrame" .. i].messageTypeList[k] = nil
					break
				end
			end
		end

		if ChatFrameEditBox.chatType == id then
			ChatFrameEditBox.chatType = "SAY"
			ChatFrameEditBox.stickyType = "SAY"
		end

		self.registered[id] = nil
	end
end

function PrivateChat:RiseChatEvent(id, message, sender, channel)
	local t_event, t_this, t_arg1, t_arg2, t_arg3, t_arg4, t_arg5, t_arg6 = event, this, arg1, arg2, arg3, arg4, arg5, arg6

	event = "CHAT_MSG_" .. id
	arg1, arg2, arg3, arg4, arg5, arg6 = message, sender, "", channel, "", ""

	for i = 1, NUM_CHAT_WINDOWS, 1 do
		for _, v in pairs(_G["ChatFrame" .. i].messageTypeList) do
			if (v == id) then
				this = _G["ChatFrame" .. i]
				ChatFrame_OnEvent(event)
			end
		end
	end

	event, this, arg1, arg2, arg3, arg4, arg5, arg6 = t_event, t_this, t_arg1, t_arg2, t_arg3, t_arg4, t_arg5, t_arg6
end

-------------------------------------------------------------------------------
-- Communication
-------------------------------------------------------------------------------

function PrivateChat:OnCommReceive(prefix, sender, distribution, messageType, data)
	if messageType == "MSG" then
		if self.registered[data.i] and self.db.profile.channels[data.i] then
			self:RiseChatEvent(data.i, tea:Decrypt(data.t, self.db.profile.channels[data.i].key), sender, self.db.profile.channels[data.i].name)
		end
	end
end
